$LOAD_PATH.unshift File.join(File.dirname(__FILE__), 'lib')

require 'open-uri'

require 'rails/generators'
require 'rails/generators/rails/app/app_generator'

require 'fast_track/ext'
require 'fast_track/dsl'
require 'fast_track/exceptions'
require 'fast_track/config'

require 'fast_track/track_methods'
require 'fast_track/track'

require 'fast_track/commands'
require 'fast_track/generator'

module FastTrack
  class App
    def initialize(app_name, tracks)
      FastTrack.options[:tracks] = tracks
      @app ||= FastTrack::Generator.new([app_name])
      @app.invoke
    end
  end

  class << self
    def options
      @options ||= {}
    end

    def tracks
      @tracks ||= make_tracks
    end

    def available_tracks
      @available_tracks ||= {}
    end

    def find_track(name)
      if available_tracks[name].present?
        available_tracks[name]
      else
        raise TrackNotFound.new(name)
      end
    end

    def config
      @config ||= FastTrack::Config.new
    end

    def root
      File.expand_path '../..', __FILE__
    end

    def template_root
      File.join(File.expand_path(root), "lib", "templates")
    end

    protected

    def make_tracks
      options[:tracks].map do |track|
        klass = FastTrack.find_track(track)
        if klass <= Track
          klass
        else
          raise TrackNotFound.new(klass.name)
        end
      end
    end
  end
end

# Load FastTrack tracks
Dir[File.join(File.dirname(__FILE__), 'tracks', "*.rb")].each do |file|
  require file
end

# Load local tracks
Dir[File.join(File.expand_path("~/"), ".fast_track", "*.rb")].each do |file|
  require file
end
