module FastTrack
  class GemInstaller
    def run(generator)
      FastTrack.tracks.each do |klass|
        unless klass.gemfile_block.blank?
          generator.instance_eval(&klass.gemfile_block)
        end
      end
    end
  end

  class BeforeMigrate
    def run(generator)
      FastTrack.tracks.each do |klass|
        unless klass.before_migrate_block.blank?
          generator.instance_eval(&klass.before_migrate_block)
        end
      end
    end
  end

  class AfterBundle
    def run(generator)
      FastTrack.tracks.each do |klass|
        unless klass.after_bundle_block.blank?
          generator.instance_eval(&klass.after_bundle_block)
        end
      end
    end
  end

  class AfterMigrate
    def run(generator)
      FastTrack.tracks.each do |klass|
        unless klass.after_migrate_block.blank?
          generator.instance_eval(&klass.after_migrate_block)
        end
      end
    end
  end

  class Runner
    def run(generator)
      FastTrack.tracks.each do |klass|
        if klass.instance_methods.include? :invoke
          klass.new(generator).invoke
        end
      end
    end
  end

  class Generator < Rails::Generators::AppGenerator
    source_root Rails::Generators::AppGenerator.source_root

    def fast_track
      FastTrack::GemInstaller.new.run(self)
      run("bundle install")
      FastTrack::AfterBundle.new.run(self)

      FastTrack::Runner.new.run(self)

      FastTrack::BeforeMigrate.new.run(self)
      rake("db:migrate")
      FastTrack::AfterMigrate.new.run(self)
    end

    protected

    def do_whatever
      puts "Does thor call this?"
    end
  end
end
