module FastTrack
  module TrackMethods
    # Do a `bundle install`
    def bundle_install
      run "bundle install"
    end

    def config
      ::FastTrack.config
    end

    def migration(migration_name, columns = {})
      migration_columns = columns.map {|k, v| "#{k}:#{v}" }
      generate "migration", migration_name, *migration_columns
    end

    # Do a `rake db:migrate`
    def db_migrate!
      rake "db:migrate"
    end

    # Create a model `model_name`
    def model(model_name, columns = {})
      model_columns = columns.map {|k, v| "#{k}:#{v}" }
      generate "model", model_name, *model_columns
    end

    # If you need to get the base-level generator object, it's
    # available here.
    def g
      generator
    end

    # Add a gem
    #
    # TODO: Find a way to consolidate all gems at once, so we only have to run
    # `bundle install` once.
    def gem(*args)
      g.gem(*args)
    end

    # Run rake
    def rake(*args)
      g.rake(*args)
    end

    # Run any command
    def run(*args)
      g.run(*args)
    end

    alias :bundle_install! :bundle_install

    def generate(*args)
      g.generate(*args)
    end

    # Add a route to config/routes.rb
    def route(*args)
      g.route(*args)
    end

    # Add a file
    def create_file(*args)
      g.create_file(*args)
    end

    # Create a gem group
    def gem_group(*args, &block)
      generator.gem_group(*args, &block)
    end

    # Read a file from the templates directory
    def read_file(file_name)
      File.read(File.join(FastTrack.template_root, template_directory, file_name))
    end
  end
end
